#[cfg(feature = "openapi")]
use crate::OpenapiType;

use serde::{de::DeserializeOwned, Serialize};

#[cfg(not(feature = "openapi"))]
pub trait ResourceType
{
}

#[cfg(not(feature = "openapi"))]
impl<T> ResourceType for T
{
}

#[cfg(feature = "openapi")]
pub trait ResourceType : OpenapiType
{
}

#[cfg(feature = "openapi")]
impl<T : OpenapiType> ResourceType for T
{
}


/// A type that can be used inside a response body. Implemented for every type that is
/// serializable with serde. If the `openapi` feature is used, it must also be of type
/// `OpenapiType`.
pub trait ResponseBody : ResourceType + Serialize
{
}

impl<T : ResourceType + Serialize> ResponseBody for T
{
}


/// A type that can be used inside a request body. Implemented for every type that is
/// deserializable with serde. If the `openapi` feature is used, it must also be of type
/// `OpenapiType`.
pub trait RequestBody : ResourceType + DeserializeOwned
{
}

impl<T : ResourceType + DeserializeOwned> RequestBody for T
{
}
